# Prox QR library client (examples)
Parsec QR Library examples for PNR QX29 family of a proximity readers.
Author [Dmitry Bogdan](mailto:dbogdan@mdo.ru). 
For support please contact [Support](mailto:support@parsec.ru)

## Manipulating with a library context
Structure `struct qr_context` is designated to communicate with the library.
Library user must deal only with a pointer to mentioned structure.


### Create context
Methods: *qr_new_ctx()* and *qr_default_ctx()* are responsible for context creation. 

Accepted *qr_new_ctx()* argument is a pointer to a private key.

#### Example create context example with a pre-defined key
```C++
    qr_AesKey_t qr_zero_key = {
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00};

    struct qr_context *ctx = qr_new_ctx((const qr_AesKey_t *)qr_zero_key);
``` 
In this above example we create library context with a key *qr_zero_key* filled with zeroes.


*qr_new_default_ctx()* does not accept any arguments.
It just creates context with a Parsec factory secret default key. Here is an example below.

#### Example create context with a default key
```C++
struct qr_context *ctx = qr_new_default_ctx();

    if (ctx != NULL)
    {
        // do great things
    }
```


After aquiring ctx pointer, please perform a NULL reference validation.

See a next paragraph for context destroy.


### Destroy context
Library context shall be disposed after usage with a *qr_delete_ctx()* method
(accepted argument is previously created context).

#### Destroy context example
```C++
// create context with the default key
struct qr_context *ctx = qr_new_default_ctx();

if (ctx != NULL) {
    qr_delete_ctx(ctx);
}
```

## QR code generation: 4 and 7 bytes
### QR code generation: Generate 4 byte code
Used method: *qr_GenId4()*.
Argument list: `struct qr_context *`*context*, `uint32_t` *Identifier*, `uint8_t *` *result_address*.

Please note that encrypted result is ASCII string,
so here and in all examples below we will reserve one character for a zero string termination.
Standard *printf()* method provided by C libary will print the string for us.

#### Example 4 byte code QR code creation
```C++
int example_gen4()
{
    int result = QR_RESULT_ERR;

    uint8_t encrypted_result[QR_DATA_STRUCT_LEN_ASCII + 1];
    encrypted_result[QR_DATA_STRUCT_LEN_ASCII] = 0;

    struct qr_context *ctx = qr_new_default_ctx();

    if (ctx != NULL)
    {
        result = qr_GenId4(ctx, 0x12345678, encrypted_result);
        qr_delete_ctx(ctx);
    }

    if (result == QR_RESULT_OK)
    {
        printf("encrypted with default key, id4: 0x12345678: %s\n", encrypted_result);
    }
    else
    {
        printf("failed to encrypt id4\n");
    }
    
    return result;
}
```
Expected result for this gen4 example is a string:
A387DFD6A95D6710C5CBAA84DD732CDF6806A07C3D7ADF16B7ADF0A33CC9C3E163E7502E34CE85A6E1B6F119BD61A68B

<https://duckduckgo.com/?q=QR+A387DFD6A95D6710C5CBAA84DD732CDF6806A07C3D7ADF16B7ADF0A33CC9C3E163E7502E34CE85A6E1B6F119BD61A68B&t=ffnt&ia=answer>


### QR code generation: Generate 7 byte code
Used method: *example_gen7()*.
Argument list: `struct qr_context *`*context*, `uint64_t` *Identifier*, `uint8_t *` *result_address*.
Synthax is the same as for ID4 with the Identifier size exception:
 it is *uint64_t* and high byte will be zeroed.

#### Example 7 byte QR code generation
```C++
int example_gen4()
{
    int result = QR_RESULT_ERR;

    uint8_t encrypted_result[QR_DATA_STRUCT_LEN_ASCII + 1];
    encrypted_result[QR_DATA_STRUCT_LEN_ASCII] = 0;

    struct qr_context *ctx = qr_new_default_ctx();

    if (ctx != NULL)
    {
        result = qr_GenId7(ctx, 0x123456789ABCDE, encrypted_result);
        qr_delete_ctx(ctx);
    }

    if (result == QR_RESULT_OK)
    {
        printf("encrypted with default key, id7: 0x123456789ABCDE: %s\n", encrypted_result);
    }
    else
    {
        printf("failed to encrypt id7\n");
    }

    return result;
}
```
Expected result for this gen7 example is a string:
D09DAB75599CC79FF11854EEE18D32FCCD300F4B9C901D068E91D42F76CA8920A95D8682CAB0B8D76EE0B31A3E2C53E6

Duckduck QR code:
<https://duckduckgo.com/?q=QR+D09DAB75599CC79FF11854EEE18D32FCCD300F4B9C901D068E91D42F76CA8920A95D8682CAB0B8D76EE0B31A3E2C53E6&t=ffnt&ia=answer>



### Light configuration
To perform light sensor and target light LED configuration.

Used method: *qr_GenLightConf()*.
Argument list: `struct qr_context *`*context*,
`uint8_t` *light_mode*, `uint16_t` *light_threshold*, `uint8_t` *target_mode*,`uint8_t *` *result_address*.

#### Argument light_mode accepted values are:

|Value |description|
--- |  ---
|QR_LIGHT_AUTO_BY_MOTION_SENSOR|by motion sensor|
|QR_LIGHT_ON|always on|
|QR_LIGHT_OFF|always off|
|QR_LIGHT_AUTO_BY_ILLUMINANCE|on when illuminance is lower than threshold|

#### Argument 'light_threshold' accepted values
When `light_mode` is set to `QR_LIGHT_AUTO_BY_ILLUMINANCE`,
light_threshold arguments can be set between `QR_LIGHT_THERSHOLD_LUX_LOW` (approx 100 lux) and `QR_LIGHT_THERSHOLD_LUX_HIGH` (approx 500 lux).

#### Argument target_mode accepted values

|Value |description|
--- |  ---
|QR_TARGET_AUTO|by motion sensor|
|QR_TARGET_ON|always on|
|QR_TARGET_OFF|always off|

#### Light coinfiguration example
```C++
int example_genlight() {
    int result = QR_RESULT_ERR;

    uint8_t encrypted_result[QR_DATA_STRUCT_LEN_ASCII + 1];
    encrypted_result[QR_DATA_STRUCT_LEN_ASCII] = 0;

    struct qr_context *ctx = qr_new_default_ctx();

    if (ctx != NULL)
    {
        result = qr_GenLightConf(ctx, QR_LIGHT_AUTO_BY_ILLUMINANCE, 300, QR_TARGET_AUTO, encrypted_result);
        qr_delete_ctx(ctx);
    }

    if (result == QR_RESULT_OK)
    {
        printf("encrypted light config: %s\n", encrypted_result);
    }
    else
    {
        printf("failed to encrypt light config\n");
    }
    
    return result;
}
```

Expected result for this set light example is a string: 
1AE2BA9B78EB77D2DBA68693B067AE9113C7CA9CD361A40F82C18BC1733EAC27115E846113F06CAA69A1CDEDCF4FB5C7

Duckduck QR code:
<https://duckduckgo.com/?q=QR+1AE2BA9B78EB77D2DBA68693B067AE9113C7CA9CD361A40F82C18BC1733EAC27115E846113F06CAA69A1CDEDCF4FB5C7&t=ffnt&ia=answer>

### QR code generation Set key to a new one
When one wish to change key to a new ona (to perform a default key replacement, *qr_GenSetAesKey()*
 method shall be used. 
Library context shall be initialized with a Proximity reader key (context with a default key is used here below).

#### Set key to a new one example
Here is the example of resetting default key to a key filled with a hexademical zeroes.
```C++
int example_gensetkey()
{
    int result = QR_RESULT_ERR;

    qr_AesKey_t qr_zero_key = {
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00};

    uint8_t encrypted_result[QR_DATA_STRUCT_LEN_ASCII + 1];
    encrypted_result[QR_DATA_STRUCT_LEN_ASCII] = 0;

    struct qr_context *ctx = qr_new_default_ctx();

    if (ctx != NULL)
    {
        result = qr_GenSetAesKey(ctx, (const qr_AesKey_t *)qr_zero_key, encrypted_result);
        qr_delete_ctx(ctx);
    }

    if (result == QR_RESULT_OK)
    {
        printf("set default to zero key: %s\n", encrypted_result);
    }
    else
    {
        printf("failed to encrypt zero key\n");
    }

    return result;
}
```
Expected result for this setkey example is a string: 6D9BB6C6AC6C72A7A25AE7A93B75403D39D8BC42CDEF651B66CD47F12E7A8878B0AECC3814A248B789D4F3AEF13C1F3E

Duckduck QR code:
<https://duckduckgo.com/?q=QR+6D9BB6C6AC6C72A7A25AE7A93B75403D39D8BC42CDEF651B66CD47F12E7A8878B0AECC3814A248B789D4F3AEF13C1F3E&t=ffnt&ia=answer>

### QR code generation Reset key to default

When one wish to change key to default (to perform a key reset), *qr_GenSetDefaultKey()* method shall be used.
Library context shall be initialized with a Proximity reader key (filled with zeroes for example).

#### Reset key from zero to default
Here is the example of resetting zero key to default key:
```C++
int example_gensetdefaultkey()
{
    int result = QR_RESULT_ERR;

    qr_AesKey_t qr_zero_key = {
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00};

    uint8_t encrypted_result[QR_DATA_STRUCT_LEN_ASCII + 1];
    encrypted_result[QR_DATA_STRUCT_LEN_ASCII] = 0;

    struct qr_context *ctx = qr_new_ctx((const qr_AesKey_t *)qr_zero_key);

    if (ctx != NULL)
    {
        result = qr_GenSetDefaultAesKey(ctx, encrypted_result);
        qr_delete_ctx(ctx);
    }

    if (result == QR_RESULT_OK)
    {
        printf("reset zero key to default: %s\n", encrypted_result);
    }
    else
    {
        printf("failed to reset zero key to default\n");
    }

    return result;
}
```
Expected result for this setdefkey example is a string:  
A60040368F15F3D9912297C4E80E190345B2A555231E5AE8EBC3EEBAB6944F11910AD590B1248816DA6DEB331C350C00

Duckduck QR code:
<https://duckduckgo.com/?q=QR+A60040368F15F3D9912297C4E80E190345B2A555231E5AE8EBC3EEBAB6944F11910AD590B1248816DA6DEB331C350C00&t=ffnt&ia=answer>


## QR code generation resources
### Duckduck search engine
<https://duckduckgo.com>

### QR-code-generator
<https://www.qr-code-generator.com/>

### The QR code generator
<https://www.the-qrcode-generator.com/>